@@ -32,17 +32,20 @@ preferred_ports = [
     '*Hex_*',
     '*Holybro_*',
     '*mRo*',
-    '*FMU*']
+    '*FMU*',
+    '*Swift-Flyer*',
+]
 
 class LinkModule(mp_module.MPModule):
 
     def __init__(self, mpstate):
         super(LinkModule, self).__init__(mpstate, "link", "link control", public=True, multi_vehicle=True)
         self.add_command('link', self.cmd_link, "link control",
-                         ["<list|ports>",
+                         ["<list|ports|resetstats>",
                           'add (SERIALPORT)',
                           'attributes (LINK) (ATTRIBUTES)',
-                          'remove (LINKS)'])
+                          'remove (LINKS)',
+                          'hl (HLSTATE)'])
         self.add_command('vehicle', self.cmd_vehicle, "vehicle control")
         self.add_command('alllinks', self.cmd_alllinks, "send command on all links")
         self.no_fwd_types = set()
@@ -50,8 +53,12 @@ class LinkModule(mp_module.MPModule):
         self.add_completion_function('(SERIALPORT)', self.complete_serial_ports)
         self.add_completion_function('(LINKS)', self.complete_links)
         self.add_completion_function('(LINK)', self.complete_links)
+        self.add_completion_function('(HLSTATE)', self.complete_hl)
         self.last_altitude_announce = 0.0
         self.vehicle_list = set()
+        self.high_latency = False
+        self.old_streamrate = 0
+        self.old_streamrate2 = 0
 
         self.menu_added_console = False
         if mp_util.has_wxpython:
@@ -88,6 +95,10 @@ class LinkModule(mp_module.MPModule):
         ports = mavutil.auto_detect_serial(preferred_list=preferred_ports)
         return [ p.device for p in ports ]
 
+    def complete_hl(self, text):
+        '''return list of hl options'''
+        return [ 'on', 'off' ]
+        
     def complete_links(self, text):
         '''return list of links'''
         try:
@@ -106,6 +117,8 @@ class LinkModule(mp_module.MPModule):
             self.show_link()
         elif args[0] == "list":
             self.cmd_link_list()
+        elif args[0] == "hl":
+            self.cmd_hl(args[1:])
         elif args[0] == "add":
             if len(args) != 2:
                 print("Usage: link add LINK")
@@ -126,9 +139,74 @@ class LinkModule(mp_module.MPModule):
                 print("Usage: link remove LINK")
                 return
             self.cmd_link_remove(args[1:])
+        elif args[0] == "resetstats":
+            self.reset_link_stats()
         else:
-            print("usage: link <list|add|remove|attributes>")
-
+            print("usage: link <list|add|remove|attributes|hl|resetstats>")
+            
+    def cmd_hl(self, args):
+        '''Toggle high latency mode'''
+        if len(args) < 1:
+            print("High latency mode is " + str(self.high_latency))
+            return
+        elif args[0] == "on": 
+            print("High latency mode ON")
+            self.high_latency = True
+            # Tell ArduPilot to start sending HIGH_LATENCY2 messages
+            self.master.mav.command_long_send(
+                self.target_system,  # target_system
+                self.target_component,
+                mavutil.mavlink.MAV_CMD_SET_MESSAGE_INTERVAL, # command
+                0, # confirmation
+                mavutil.mavlink.MAVLINK_MSG_ID_HIGH_LATENCY2, # param1 (msg id)
+                1000000, # param2  (message interval, us)
+                0, # param3
+                0, # param4
+                0, # param5
+                0, # param6
+                0) # param7
+            # and stop sending any other messages
+            self.old_streamrate = self.settings.streamrate
+            self.old_streamrate2 = self.settings.streamrate2
+            self.settings.streamrate = -1
+            self.settings.streamrate2 = -1
+            for master in self.mpstate.mav_master:
+                master.mav.request_data_stream_send(self.mpstate.settings.target_system, self.mpstate.settings.target_component,
+                                                    mavutil.mavlink.MAV_DATA_STREAM_ALL,
+                                                    0, 1)
+            return
+        elif args[0] == "off": 
+            print("High latency mode OFF")
+            self.high_latency = False
+            # Start sending the full message set again
+            self.settings.streamrate = self.old_streamrate
+            self.settings.streamrate2 = self.old_streamrate2
+            for master in self.mpstate.mav_master:
+                if master.linknum == 0:
+                    rate = self.settings.streamrate
+                else:
+                    rate = self.settings.streamrate2
+                if rate != -1 and self.mpstate.settings.streamrate != -1:
+                    master.mav.request_data_stream_send(self.mpstate.settings.target_system, self.mpstate.settings.target_component,
+                                                        mavutil.mavlink.MAV_DATA_STREAM_ALL,
+                                                        rate, 1)
+            # Tell ArduPilot to stop sending HIGH_LATENCY2 messages
+            self.master.mav.command_long_send(
+                self.target_system,  # target_system
+                self.target_component,
+                mavutil.mavlink.MAV_CMD_SET_MESSAGE_INTERVAL, # command
+                0, # confirmation
+                mavutil.mavlink.MAVLINK_MSG_ID_HIGH_LATENCY2, # param1 (msg id)
+                -1, # param2  (message interval)
+                0, # param3
+                0, # param4
+                0, # param5
+                0, # param6
+                0) # param7
+            return
+        else:
+            print("usage: hl <on|off>")
+                        
     def show_link(self):
         '''show link information'''
         for master in self.mpstate.mav_master:
@@ -160,6 +238,15 @@ class LinkModule(mp_module.MPModule):
                                                                                     self.status.bytecounters['MasterIn'][master.linknum].rate(),
                                                                                     sign_string))
 
+
+    def reset_link_stats(self):
+        '''reset link statistics'''
+        for master in self.mpstate.mav_master:
+            self.status.counters['MasterIn'][master.linknum] = 0
+            self.status.bytecounters['MasterIn'][master.linknum].__init__()
+            master.mav_loss = 0
+            master.mav_count = 0
+
     def cmd_alllinks(self, args):
         '''send command on all links'''
         saved_target = self.mpstate.settings.target_system
@@ -348,6 +435,8 @@ class LinkModule(mp_module.MPModule):
             return
 
         msec = m.time_boot_ms
+        if msec == 0:
+            return
         sysid = m.get_srcSystem()
         compid = m.get_srcComponent()
         highest_msec_key = (sysid,compid)
@@ -441,7 +530,7 @@ class LinkModule(mp_module.MPModule):
             
         mtype = m.get_type()
 
-        if mtype == 'HEARTBEAT' and m.type != mavutil.mavlink.MAV_TYPE_GCS:
+        if (mtype == 'HEARTBEAT' or mtype == 'HIGH_LATENCY2') and m.type != mavutil.mavlink.MAV_TYPE_GCS:
             if self.settings.target_system == 0 and self.settings.target_system != m.get_srcSystem():
                 self.settings.target_system = m.get_srcSystem()
                 self.say("online system %u" % self.settings.target_system,'message')
@@ -659,7 +748,7 @@ class LinkModule(mp_module.MPModule):
         '''handle an incoming mavlink packet'''
         type = msg.get_type()
 
-        if type == 'HEARTBEAT':
+        if type == 'HEARTBEAT' or type == 'HIGH_LATENCY2':
             sysid = msg.get_srcSystem()
             if not sysid in self.vehicle_list and msg.type != mavutil.mavlink.MAV_TYPE_GCS:
                 self.vehicle_list.add(sysid)
