@@ -30,6 +30,7 @@ from MAVProxy.modules.mavproxy_map.mp_slipmap_util import SlipFollowObject
 
 from MAVProxy.modules.lib import mp_util
 from MAVProxy.modules.lib import win_layout
+from MAVProxy.modules.lib import icon
 
 from MAVProxy.modules.lib.mp_menu import MPMenuCheckbox
 from MAVProxy.modules.lib.mp_menu import MPMenuItem
@@ -44,6 +45,7 @@ class MPSlipMapFrame(wx.Frame):
     """
     def __init__(self, state):
         wx.Frame.__init__(self, None, wx.ID_ANY, state.title)
+        self.SetIcon(icon.SimpleIcon("MAP").get_ico())
         self.state = state
         state.frame = self
         state.grid = True
@@ -386,6 +388,10 @@ class MPSlipMapPanel(wx.Panel):
         state = self.state
         return state.mt.coord_from_area(x, y, state.lat, state.lon, state.width, state.ground_width)
 
+    def constrain_latlon(self):
+        self.state.lat = mp_util.constrain(self.state.lat, -85, 85)
+        self.state.lon = mp_util.wrap_180(self.state.lon)
+
     def re_center(self, x, y, lat, lon):
         '''re-center view for pixel x,y'''
         state = self.state
@@ -395,6 +401,7 @@ class MPSlipMapPanel(wx.Panel):
         distance = mp_util.gps_distance(lat2, lon2, lat, lon)
         bearing  = mp_util.gps_bearing(lat2, lon2, lat, lon)
         (state.lat, state.lon) = mp_util.gps_newpos(state.lat, state.lon, bearing, distance)
+        self.constrain_latlon()
 
     def set_ground_width(self, ground_width):
         '''set ground width of view'''
@@ -410,6 +417,8 @@ class MPSlipMapPanel(wx.Panel):
         else:
             (x,y) = (state.width/2, state.height/2)
         (lat,lon) = self.coordinates(x, y)
+        lat = mp_util.constrain(lat, -85, 85)
+        lon = mp_util.constrain(lon, -180, 180)
         state.ground_width *= zoom
         # limit ground_width to sane values
         state.ground_width = max(state.ground_width, 2)
@@ -426,6 +435,7 @@ class MPSlipMapPanel(wx.Panel):
             dlg.Destroy()
             state.lat = float(latlon[0])
             state.lon = float(latlon[1])
+            self.constrain_latlon()
             self.re_center(state.width/2,state.height/2, state.lat, state.lon)
             self.redraw_map()
 
@@ -510,7 +520,7 @@ class MPSlipMapPanel(wx.Panel):
             
         # find display bounding box
         (lat2,lon2) = self.coordinates(state.width-1, state.height-1)
-        bounds = (lat2, state.lon, state.lat-lat2, lon2-state.lon)
+        bounds = (lat2, state.lon, state.lat-lat2, mp_util.wrap_180(lon2-state.lon))
 
         # get the image
         img = self.map_img.copy()
@@ -661,6 +671,7 @@ class MPSlipMapPanel(wx.Panel):
                     distance = (state.ground_width/float(state.width)) * pdist
                     newlatlon = mp_util.gps_newpos(state.lat, state.lon, bearing, distance)
                     (state.lat, state.lon) = newlatlon
+                    self.constrain_latlon()
                     self.mouse_down = newpos
                     self.redraw_map()
 
