@@ -76,6 +76,7 @@ class MiscModule(mp_module.MPModule):
         self.add_command('time', self.cmd_time, "show autopilot time")
         self.add_command('shell', self.cmd_shell, "run shell command")
         self.add_command('changealt', self.cmd_changealt, "change target altitude")
+        self.add_command('changealt_abs', self.cmd_changealt_abs, "change target absolute altitude")
         self.add_command('land', self.cmd_land, "auto land")
         self.add_command('repeat', self.cmd_repeat, "repeat a command at regular intervals",
                          ["<add|remove|clear>"])
@@ -93,10 +94,12 @@ class MiscModule(mp_module.MPModule):
         self.add_command('hardfault_autopilot', self.cmd_hardfault_autopilot, "hardfault autopilot")
         self.add_command('panic_autopilot', self.cmd_panic_autopilot, "panic autopilot")
         self.add_command('longloop_autopilot', self.cmd_longloop_autopilot, "cause long loop in autopilot")
+        self.add_command('internalerror_autopilot', self.cmd_internalerror_autopilot, "cause internal error in autopilot")
         self.add_command('batreset', self.cmd_battery_reset, "reset battery remaining")
         self.add_command('setorigin', self.cmd_setorigin, "set global origin")
         self.add_command('magsetfield', self.cmd_magset_field, "set expected mag field by field")
         self.add_command('magresetofs', self.cmd_magreset_ofs, "reset offsets for all compasses")
+        self.add_command('namedvaluefloat', self.cmd_namedvaluefloat, "send a NAMED_VALUE_FLOAT")
         self.repeats = []
 
     def altitude_difference(self, pressure1, pressure2, ground_temp):
@@ -192,6 +195,10 @@ class MiscModule(mp_module.MPModule):
         '''Ask the autopilot to create a long loop'''
         self.cmd_dosomethingreallynastyto_autopilot(args, 'long-loop', 97)
 
+    def cmd_internalerror_autopilot(self, args):
+        '''Ask the autopilot to create an internal error'''
+        self.cmd_dosomethingreallynastyto_autopilot(args, 'internal-error', 98)
+
     def cmd_battery_reset(self, args):
         '''reset battery remaining'''
         mask = -1
@@ -227,6 +234,21 @@ class MiscModule(mp_module.MPModule):
                                           0, 0, relalt)
         print("Sent change altitude command for %.1f meters" % relalt)
 
+    def cmd_changealt_abs(self, args):
+        '''change target altitude'''
+        if len(args) < 1:
+            print("usage: changealt <relaltitude>")
+            return
+        absalt = float(args[0])
+        self.master.mav.mission_item_send(self.settings.target_system,
+                                          self.settings.target_component,
+                                          0,
+                                          0,
+                                          mavutil.mavlink.MAV_CMD_NAV_WAYPOINT,
+                                          3, 1, 0, 0, 0, 0,
+                                          0, 0, absalt)
+        print("Sent change altitude command for %.1f meters" % absalt)
+        
     def cmd_land(self, args):
         '''auto land commands'''
         if len(args) < 1:
@@ -376,6 +398,8 @@ class MiscModule(mp_module.MPModule):
                 mp_util.decode_devid(self.mav_param[p], p)
             if p.startswith('BARO') and p.endswith('_DEVID'):
                 mp_util.decode_devid(self.mav_param[p], p)
+            if p.startswith('ARSPD') and p.endswith('_DEVID'):
+                mp_util.decode_devid(self.mav_param[p], p)
 
     def cmd_setorigin(self, args):
         '''set global origin'''
@@ -456,7 +480,16 @@ class MiscModule(mp_module.MPModule):
         self.param_set('COMPASS_ODI3_X', 0)
         self.param_set('COMPASS_ODI3_Y', 0)
         self.param_set('COMPASS_ODI3_Z', 0)
-        
+
+    def cmd_namedvaluefloat(self, args):
+        '''send a NAMED_VALUE_FLOAT'''
+        if len(args) < 2:
+            print("Usage: namedvaluefloat NAME value")
+            return
+        tnow_ms = int((time.time() - self.mpstate.start_time_s)*1000)
+        name = args[0]
+        value = float(args[1])
+        self.master.mav.named_value_float_send(tnow_ms, name.encode("utf-8"), value)
 
     def idle_task(self):
         '''called on idle'''
