@@ -14,10 +14,12 @@ import os
 import fnmatch
 import threading
 import shlex
+import traceback
 from math import *
 from MAVProxy.modules.lib import multiproc
 from MAVProxy.modules.lib import rline
 from MAVProxy.modules.lib import wxconsole
+from MAVProxy.modules.lib import param_help
 from MAVProxy.modules.lib.graph_ui import Graph_UI
 from pymavlink.mavextra import *
 from MAVProxy.modules.lib.mp_menu import *
@@ -26,6 +28,7 @@ from pymavlink import mavutil
 from MAVProxy.modules.lib.mp_settings import MPSettings, MPSetting
 from MAVProxy.modules.lib import wxsettings
 from MAVProxy.modules.lib.graphdefinition import GraphDefinition
+from MAVProxy.modules.lib import icon
 from lxml import objectify
 import pkg_resources
 from builtins import input
@@ -86,7 +89,7 @@ class MEState(object):
     def __init__(self):
         self.input_queue = multiproc.Queue()
         self.rl = None
-        self.console = wxconsole.MessageConsole(title='MAVExplorer')
+        self.console = wxconsole.MessageConsole(title='MAVExplorer', ico=icon.SimpleIcon("EXPLORER"))
         self.exit = False
         self.status = MEStatus()
         self.settings = MPSettings(
@@ -100,17 +103,20 @@ class MEState(object):
               MPSetting('legend', str, 'upper left', 'legend position'),
               MPSetting('legend2', str, 'upper right', 'legend2 position'),
               MPSetting('title', str, None, 'Graph title'),
+              MPSetting('debug', int, 0, 'debug level'),
               ]
             )
 
         self.mlog = None
+        self.mav_param = None
         self.filename = None
         self.command_map = command_map
         self.completions = {
             "set"       : ["(SETTING)"],
             "condition" : ["(VARIABLE)"],
             "graph"     : ['(VARIABLE) (VARIABLE) (VARIABLE) (VARIABLE) (VARIABLE) (VARIABLE) (VARIABLE) (VARIABLE) (VARIABLE) (VARIABLE) (VARIABLE) (VARIABLE)'],
-            "map"       : ['(VARIABLE) (VARIABLE) (VARIABLE) (VARIABLE) (VARIABLE)']
+            "map"       : ['(VARIABLE) (VARIABLE) (VARIABLE) (VARIABLE) (VARIABLE)'],
+            "param"     : ['download', 'check', 'help (PARAMETER)'],
             }
         self.aliases = {}
         self.graphs = []
@@ -121,6 +127,7 @@ class MEState(object):
         self.parent_pipe_recv_console,self.child_pipe_send_console = multiproc.Pipe(duplex=False)
         #pipe for creating graphs (such as from the save dialog box)
         self.parent_pipe_recv_graph,self.child_pipe_send_graph = multiproc.Pipe(duplex=False)
+        self.param_help = param_help.ParamHelp()
         
         tConsoleWrite = threading.Thread(target=self.pipeRecvConsole)
         tConsoleWrite.daemon = True
@@ -128,7 +135,7 @@ class MEState(object):
         tGraphWrite = threading.Thread(target=self.pipeRecvGraph)
         tGraphWrite.daemon = True
         tGraphWrite.start()
-                
+
     def pipeRecvConsole(self):
         '''watch for piped data from save dialog'''
         try:
@@ -284,7 +291,11 @@ def load_graph_xml(xml, filename, load_all=False):
         expressions = [e.text for e in g.expression]
         if load_all:
             if not name in names:
-                ret.append(GraphDefinition(name, expressions[0], g.description.text, expressions, filename))
+                if hasattr(g,'description'):
+                    description = g.description.text
+                else:
+                    description = ''
+                ret.append(GraphDefinition(name, expressions[0], description, expressions, filename))
             names.add(name)
             continue
         if have_graph(name):
@@ -292,7 +303,11 @@ def load_graph_xml(xml, filename, load_all=False):
         for e in expressions:
             e = xml_unescape(e)
             if expression_ok(e):
-                ret.append(GraphDefinition(name, e, g.description.text, expressions, filename))
+                if hasattr(g,'description'):
+                    description = g.description.text
+                else:
+                    description = ''
+                ret.append(GraphDefinition(name, e, description, expressions, filename))
                 break
     return ret
 
@@ -368,6 +383,8 @@ def cmd_graph(args):
     else:
         expression = ' '.join(args)
         mestate.last_graph = GraphDefinition(mestate.settings.title, expression, '', [expression], None)
+    if mestate.settings.debug > 0:
+        print("Adding graph: %s" % mestate.last_graph.expression)
     grui.append(Graph_UI(mestate))
     grui[-1].display_graph(mestate.last_graph, flightmode_colours())
     global xlimits
@@ -638,6 +655,10 @@ events = {
     71 : "DATA_ZIGZAG_STORE_A",
     72 : "DATA_ZIGZAG_STORE_B",
     73 : "DATA_LAND_REPO_ACTIVE",
+
+    80 : "FENCE_FLOOR_ENABLE",
+    81 : "FENCE_FLOOR_DISABLE",
+
     163 : "DATA_SURFACED",
     164 : "DATA_NOT_SURFACED",
     165 : "DATA_BOTTOMED",
@@ -746,16 +767,45 @@ def cmd_messages(args):
             print("%s %s" % (timestring(m), mstr))
     mestate.mlog.rewind()
 
+def set_vehicle_name():
+    mapping = { mavutil.mavlink.MAV_TYPE_GROUND_ROVER : "Rover",
+                mavutil.mavlink.MAV_TYPE_FIXED_WING : "ArduPlane",
+                mavutil.mavlink.MAV_TYPE_QUADROTOR : "ArduCopter",
+                mavutil.mavlink.MAV_TYPE_ANTENNA_TRACKER : "AntennaTracker",
+                mavutil.mavlink.MAV_TYPE_SUBMARINE : "ArduSub",
+                }
+    mestate.param_help.vehicle_name = mapping.get(mestate.mlog.mav_type, None)
+
 def cmd_param(args):
     '''show parameters'''
+    verbose = False
     if len(args) > 0:
+        if args[0] == 'help':
+            set_vehicle_name()
+            mestate.param_help.param_help(args[1:])
+            return
+        if args[0] == 'download':
+            mestate.param_help.param_help_download()
+            return
+        if args[0] == 'check':
+            set_vehicle_name()
+            mestate.param_help.param_check(mestate.mlog.params, args[1:])
+            return
         wildcard = args[0]
+        if len(args) > 1 and args[1] == "-v":
+            set_vehicle_name()
+            verbose = True
     else:
         wildcard = '*'
     k = sorted(mestate.mlog.params.keys())
     for p in k:
         if fnmatch.fnmatch(str(p).upper(), wildcard.upper()):
-            print("%-16.16s %f" % (str(p), mestate.mlog.params[p]))
+            s = "%-16.16s %f" % (str(p), mestate.mlog.params[p])
+            if verbose:
+                info = mestate.param_help.param_info(p, mestate.mlog.params[p])
+                if info is not None:
+                    s += " # %s" % info
+            print(s)
 
 def cmd_paramchange(args):
     '''show param changes'''
@@ -805,6 +855,8 @@ def cmd_devid(args):
             mp_util.decode_devid(params[p], p)
         if p.startswith('BARO') and p.endswith('DEVID'):
             mp_util.decode_devid(params[p], p)
+        if p.startswith('ARSPD') and p.endswith('DEVID'):
+            mp_util.decode_devid(params[p], p)
 
 def cmd_loadfile(args):
     '''callback from menu to load a log file'''
@@ -850,6 +902,8 @@ def loadfile(args):
     global flightmodes
     flightmodes = mlog.flightmode_list()
 
+    mestate.mav_param = mlog.params
+
     setup_menus()
 
 def process_stdin(line):
@@ -881,6 +935,15 @@ def process_stdin(line):
         fn(args[1:])
     except Exception as e:
         print("ERROR in command %s: %s" % (args[1:], str(e)))
+        if mestate.settings.debug > 0:
+            if sys.version_info[0] >= 3:
+                ret = "%s\n" % e
+                ret += ''.join(traceback.format_exception(etype=type(e),
+                                                        value=e,
+                                                        tb=e.__traceback__))
+                print(ret)
+            else:
+                print(traceback.format_exc(e))
 
 def input_loop():
     '''wait for user input'''
